<?php
/*
Plugin Name: Advanced TinyMCE Editor
Plugin URI: http://www.mkbergman.com/advancedTinyMCE
Description: Adds advanced features to the standard TinyMCE rich-text editor. Originally by Assaf Arkin, now updated many times.  Not tested prior to WP v 2.2.  Control panel by Chris Carson (chris@navyroad.com) allows stylesheet entry and buttons display.  After activation, go to Options > Advanced TinyMCE.
Version: 0.5.0
Date: 7/16/2007
Author: Michael K. Bergman
Author URI: http://www.mkbergman.com/
License: Creative Commons Attribution-ShareAlike
Tags: wordpress tinymce
*/

/*
 * Changes by chris carson chris@navyroad.com (mostly) for version 0.5.0:
 * 
 * -- encapsulated plugin code in a class
 * -- added control panel under "Options"
 * 	-- an input to enter the urls of content stylesheets, separated by commas
 * 	-- for each row of buttons
 * 		-- for each button group
 * 			-- checkboxes for each button
 * -- added WordPress option "Advanced_TinyMCE_Editor": associative array:
 * 	-- css: string containing stylesheet URL(s), separated by commas
 * 	-- btns: an array of buttons in the format $this->options["btns"][$row][$group][$btn] => $label
 * -- added explanation about editor width
 * -- see http://www.mkbergman.com/?page_id=383 for a description of button functions, other documentation
 *
 * --TODO
 * 	-- validate stylesheet URLs
 */

class Advanced_TinyMCE_Editor{
	var $namespace = "Advanced_TinyMCE_Editor";
	var $arr_plugins;
	var $arr_btns;
	var $options;
	function Advanced_TinyMCE_Editor(){
		$this->arr_plugins =  array(
		"advcolor", "advhr", "advimage", "advlink",
		"cleanup", "contextmenu", "directionality", "emotions",
		"fullscreen", "iespell", "insertdatetime", "media",
		"print", "searchreplace", "style", "table",
      "visualchars", "wphelp", "xhtmlxtras");


		$this->arr_btns = array(
			array(
				array( "undo", "redo"),
				array( "print"),
				array( "cut", "copy", "paste","pastetext", "pasteword","selectall"),
				array( "formatselect","fontselect", "fontsizeselect", "styleselect", "styleprops"),
				array( "code")
			),
			array(
				array( "bold", "italic", "underline", "strikethrough"),
				array( "bullist", "numlist"),
				array( "justifyleft", "justifycenter", "justifyright", "justifyfull"),
				array( "indent", "outdent"),
				array( "forecolor", "backcolor", "advcolor"),
				array( "sub", "sup", "charmap", "emotions"),
				array( "insertdate", "inserttime"),
				array( "hr", "advhr"),
				array( "image", "media"),
				array( "link", "unlink", "anchor")

			),
			array(
				array( "tablecontrols"),
				array( "cite", "abbr", "acronymn", "ins", "del", "attribs"),
				array( "ltr", "rtl"),
				array( "iespell", "spellchecker"),
				array( "visualchars", "removeformat","cleanup"),
				array( "search", "replace"),
				array( "fullscreen"),
				array( "wp_more", "wp_help")
			)
		);

		add_action("plugins_loaded",array($this,"init"));

	}
	function init(){
		global $wp_version;
		$this->options = get_option($this->namespace);

		if (isset($wp_version)) {
			add_action('admin_menu', array($this,"add_admin"));
			add_filter("mce_plugins", array($this,"extended_editor_mce_plugins"), 0);
			add_filter("mce_buttons", array($this,"extended_editor_mce_buttons"), 0);
			add_filter("mce_buttons_2", array($this,"extended_editor_mce_buttons_2"), 0);
			add_filter("mce_buttons_3", array($this,"extended_editor_mce_buttons_3"), 0);
			// added on 6/04/07; see accompanying function
			add_filter("mce_valid_elements", array($this,"extended_editor_mce_valid_elements"),0);
			add_filter("mce_css", array($this,"extended_editor_mce_css"),0);

		}
	}
	function extended_editor_mce_css($css){
		if (! empty($this->options["css"])){
			return implode(",", array($css, $this->options["css"]));
		}
		return $css;
	}
	function add_admin(){
		add_options_page("Advanced TinyMCE","Advanced TinyMCE",10,$this->namespace, array($this,"control_panel"));
	}

	function extended_editor_mce_plugins($plugins) {
		if (! is_array($this->options["plugins"])) return   array_merge($this->arr_plugins, $plugins);
		else return   array_merge($this->options["plugins"], $plugins);
	}

	function extended_editor_mce_buttons($buttons) {
		return $this->get_btn_row_array(0);
	}

	function extended_editor_mce_buttons_2($buttons) {
		// the second toolbar line
		return $this->get_btn_row_array(1);
	}

	function extended_editor_mce_buttons_3($buttons) {
		// These are the buttons for third toolbar line
		return $this->get_btn_row_array(2);
	}
	function get_btn_row_array($row){

		$group_arrays = array();
		foreach ($this->arr_btns[$row] as $group=>$btns){
			$group_array = array();
			foreach ($btns as $btn=>$label) {
				if (in_array($label, $this->options["btns"][$row][$group])){
					$group_array[] = $label;
				}
			}
			if (count($group_array)) $group_arrays[] = $group_array;
		}
		$returned = array();
		foreach ($group_arrays as $ind=>$group_array) {
			$returned = array_merge($returned, $group_array);
			if ($ind < count($group_arrays) - 1) $returned = array_merge($returned, array("separator"));
		}
		return count($returned) ? $returned : array("separator");
	}

	function extended_editor_mce_valid_elements($valid_elements) {
		$valid_elements = '-strong/-b[*],-em/-i[*],-font[*],-ul[*],-ol[*],-li[*],*[*]';
		$valid_elements = apply_filters('extended_editor_mce_valid_elements', $valid_elements);
		return $valid_elements;
	}
	function control_panel(){
		//var_dump($this->options);
		if (! is_array($this->options["plugins"])) $this->options["plugins"] = $this->arr_plugins;
		if (! is_array($this->options["btns"])) $this->options["btns"] = $this->arr_btns;
		$var_submit = $this->namespace . "_submit";
		$var_css =  $this->namespace . "_css";
		$var_btns =  $this->namespace . "_btns";
		$message = "";
		if ($_POST[$var_submit] == 1){
			check_admin_referer($this->namespace);

			$this->options["css"] = trim(stripslashes($_POST[$var_css]));

			foreach ($this->arr_btns as $row=>$groups){
				foreach($groups as $group=>$btns){
					foreach ($btns as $btn=>$label){
						if ($_POST[$var_btns][$row][$group][$btn] == 1){
							$this->options["btns"][$row][$group][$btn] = $label;
						} else {
							$this->options["btns"][$row][$group][$btn] = "";
						}
					}
				}
			}
			update_option($this->namespace, $this->options);
			$message = "TinyMCE options updated.";
		}

		require(dirname(__FILE__) . "/control_panel.php");
	}
}

$Advanced_TinyMCE_Editor = new Advanced_TinyMCE_Editor();